"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getEntityAttributes = void 0;
const graphql_1 = require("graphql");
const getTopFieldName_1 = require("./getTopFieldName");
const utils_1 = require("../../../../../common/utils");
const getAttributeNames_1 = require("./getAttributeNames");
const getEntityName_1 = require("./getEntityName");
const getTypeFields_1 = require("./getTypeFields");
const isAnyLeafType_1 = require("./isAnyLeafType");
const getSelectedFieldNamesFromTopField_1 = require("./getSelectedFieldNamesFromTopField");
const unwrapGraphQLType_1 = require("./unwrapGraphQLType");
const isFieldOfGraphQLNonNullType_1 = require("./isFieldOfGraphQLNonNullType");
const DATE_TYPES = ['Date', 'Timestamp', 'LocalDateTime', 'LocalTime', 'DateTime', 'Time'];
const NUMBER_TYPES = ['Int', 'BigDecimal', 'BigInteger', 'Float', 'Long'];
function getEntityAttributes(documentNode, schema, idField) {
    // e.g. "petList"
    const queryFieldName = getTopFieldName_1.getTopFieldName(documentNode);
    // e.g. "PetDTO"
    const entityName = getEntityName_1.getEntityName(queryFieldName, schema);
    const typeFields = getTypeFields_1.getTypeFields(entityName, schema);
    const selectedFieldNames = getSelectedFieldNamesFromTopField_1.getSelectedFieldNamesFromTopField(documentNode);
    if (!selectedFieldNames.find(fieldKey => typeFields[fieldKey].name === idField)) {
        throw new Error(`Attributes of query '${queryFieldName}' did not contain id attribute with name '${idField}', ` +
            `which required for screen generation. If you are using entity with id field with name distinct from 'id', ` +
            `please be sure that you pass information about id field name in answers ('idField', 'listIdField', 'detailsIdField')`);
    }
    return selectedFieldNames
        // Do not include id attribute
        .filter(fieldKey => typeFields[fieldKey].name !== idField)
        .map(fieldKey => {
        const field = typeFields[fieldKey];
        const gqlType = isFieldOfGraphQLNonNullType_1.isFieldOfGraphQLNonNullType(field)
            ? unwrapGraphQLType_1.unwrapGraphQLType(field.type)
            : field.type;
        if (gqlType == null) {
            throw new Error(`GraphQL type is null for field '${field}' in query '${queryFieldName}'`);
        }
        const namedType = graphql_1.getNamedType(gqlType);
        const type = namedType.toString();
        return {
            kind: 'gql',
            name: field.name,
            displayName: utils_1.capitalizeFirst(utils_1.splitByCapitalLetter(field.name)),
            isSingleRelationField: !isAnyLeafType_1.isAnyLeafType(gqlType) && !(gqlType instanceof graphql_1.GraphQLList),
            isMultiRelationField: !isAnyLeafType_1.isAnyLeafType(gqlType) && gqlType instanceof graphql_1.GraphQLList,
            type,
            gqlType,
            isDateType: DATE_TYPES.includes(type),
            isNumberType: NUMBER_TYPES.includes(type),
            enumOptions: gqlType instanceof graphql_1.GraphQLEnumType
                ? gqlType.getValues()
                : undefined,
            nestedAttributes: isAnyLeafType_1.isAnyLeafType(gqlType)
                ? undefined
                : getAttributeNames_1.getAttributeNames(type, schema),
        };
    });
}
exports.getEntityAttributes = getEntityAttributes;
//# sourceMappingURL=getEntityAttributes.js.map