"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getValidatorImport = exports.deriveClientValidationRules = void 0;
const graphql_1 = require("graphql");
const getInputFieldType_1 = require("../../../building-blocks/stages/template-model/pieces/graphql-utils/getInputFieldType");
const utils_1 = require("../../../common/utils");
const TEMPORAL_DIRECTIVES = [
    'futureOrPresent',
    'future',
    'past',
    'pastOrPresent',
];
const SUPPORTED_DIRECTIVES = [
    'assertFalse',
    'assertTrue',
    'decimalMax',
    'decimalMin',
    'digits',
    'email',
    'max',
    'min',
    'negative',
    'negativeOrZero',
    'notBlank',
    'notEmpty',
    'notNull',
    'null',
    'pattern',
    'positive',
    'positiveOrZero',
    'size',
    'length',
    'url',
    ...TEMPORAL_DIRECTIVES
];
function deriveClientValidationRules(node, schema) {
    if (node == null || schema == null) {
        return { clientValidationRules: {}, validatorImports: [] };
    }
    const clientValidationRules = {};
    const validatorImportsSet = new Set();
    const mutationArgumentType = node.definitions[0].variableDefinitions?.[0].type;
    if (mutationArgumentType == null) {
        throw new Error(`Cannot find input type, mutationArgumentType is ${mutationArgumentType}`);
    }
    let inputName;
    if ('type' in mutationArgumentType && 'name' in mutationArgumentType.type) {
        inputName = mutationArgumentType.type.name.value;
    }
    else if ('name' in mutationArgumentType) {
        inputName = mutationArgumentType.name.value;
    }
    else {
        throw new Error('Cannot find input type name, mutationArgumentType is ' + JSON.stringify(mutationArgumentType));
    }
    const inputType = schema.getType(inputName);
    const fields = inputType.getFields();
    Object.keys(fields).forEach(field => {
        const rules = [];
        fields[field].astNode?.directives?.forEach(directive => {
            const directiveName = directive.name.value;
            if (!SUPPORTED_DIRECTIVES.includes(directiveName)) {
                console.error(`Unsupported directive ${directiveName} on attribute ${field} of input ${inputName}`);
                return;
            }
            const rule = {
                rule: directiveName
            };
            directive.arguments?.forEach(arg => {
                if ('value' in arg.value) {
                    rule[arg.name.value] = convertDirectiveArgValue(arg.value.value, directiveName);
                }
            });
            if (isTemporalDirective(directiveName)) {
                rule.type = getTemporalRuleType(fields[field]);
            }
            if (directiveName === 'size') {
                rule.isList = graphql_1.isListType(fields[field]);
            }
            rules.push(rule);
            const validatorImport = getValidatorImport(directiveName, rule.type);
            if (validatorImport != null) {
                validatorImportsSet.add(validatorImport);
            }
        });
        // There are cases when we add validation rules based not on @directive but on Scalar type or nullability
        const scalarType = fields[field].type;
        if (scalarType instanceof graphql_1.GraphQLNonNull) { // e.g. String!
            rules.push({ rule: 'notNull' });
        }
        if (scalarType instanceof graphql_1.GraphQLScalarType && scalarType.name === 'Url') {
            rules.push({ rule: 'url' });
        }
        if (rules.length > 0) {
            clientValidationRules[field] = rules;
        }
    });
    return {
        clientValidationRules,
        validatorImports: Array.from(validatorImportsSet)
    };
}
exports.deriveClientValidationRules = deriveClientValidationRules;
function convertDirectiveArgValue(value, directiveName) {
    if (['length', 'size', 'digits', 'max', 'min'].includes(directiveName)) {
        return Number(value);
    }
    return value;
}
function isTemporalDirective(directiveName) {
    return TEMPORAL_DIRECTIVES.includes(directiveName);
}
function getTemporalRuleType(field) {
    const typeName = getInputFieldType_1.getInputFieldTypeName(field);
    const DATE_TYPES = ['Date'];
    const INSTANT_TYPES = ['Timestamp', 'DateTime', 'Time', 'LocalTime', 'LocalDateTime'];
    if (DATE_TYPES.includes(typeName)) {
        return 'date';
    }
    if (INSTANT_TYPES.includes(typeName)) {
        return 'instant';
    }
    console.warn(`Unexpected temporal type ${typeName} on field ${field.name}. Client validation may not work properly for this field.`);
    return 'instant';
}
/**
 * Returns undefined when directive doesn't require custom Antd validator
 *
 * @param directiveName
 * @param ruleType
 */
function getValidatorImport(directiveName, ruleType) {
    if (isTemporalDirective(directiveName)) {
        if (ruleType == null) {
            throw new Error(`Expected client validation rule ${directiveName} to have type 'date' or 'instant', was ${ruleType}. Cannot determine validator import.`);
        }
        return `temporal/${directiveName}${utils_1.capitalizeFirst(ruleType)}`;
    }
    if (['negative', 'negativeOrZero', 'positive', 'positiveOrZero'].includes(directiveName)) {
        return `numeric/${directiveName}`;
    }
    if (['min', 'max', 'decimalMin', 'decimalMax'].includes(directiveName)) {
        return 'numeric/compare';
    }
    if (directiveName === 'digits') {
        return 'numeric/digits';
    }
    if (['assertFalse', 'assertTrue', 'null'].includes(directiveName)) {
        return directiveName;
    }
    return undefined;
}
exports.getValidatorImport = getValidatorImport;
//# sourceMappingURL=client-validation.js.map