"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const client_validation_1 = require("./client-validation");
const chai_1 = require("chai");
const test_commons_1 = require("../../../test/test-commons");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
describe('deriveClientValidationRules()', () => {
    it('correctly derives the rules from GraphQL schema', async () => {
        const schema = await test_commons_1.getMockSchema();
        const node = graphql_tag_1.default(MUTATION);
        const { clientValidationRules, validatorImports } = client_validation_1.deriveClientValidationRules(node, schema);
        chai_1.expect(clientValidationRules).to.deep.eq(EXPECTED_RULES);
        chai_1.expect(validatorImports.sort()).to.deep.eq(EXPECTED_IMPORTS.sort());
    });
    it('Update_Owner', async () => {
        const schema = await test_commons_1.getMockSchema();
        const node = graphql_tag_1.default(OWNER_MUTATION);
        const { clientValidationRules } = client_validation_1.deriveClientValidationRules(node, schema);
        chai_1.expect(clientValidationRules).to.not.be.undefined;
    });
});
describe('getValidatorImport()', () => {
    it('correctly determines validator imports', () => {
        chai_1.expect(client_validation_1.getValidatorImport('assertFalse')).to.eq('assertFalse');
        chai_1.expect(client_validation_1.getValidatorImport('assertTrue')).to.eq('assertTrue');
        chai_1.expect(client_validation_1.getValidatorImport('decimalMax')).to.eq('numeric/compare');
        chai_1.expect(client_validation_1.getValidatorImport('decimalMin')).to.eq('numeric/compare');
        chai_1.expect(client_validation_1.getValidatorImport('digits')).to.eq('numeric/digits');
        chai_1.expect(client_validation_1.getValidatorImport('email')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('max')).to.eq('numeric/compare');
        chai_1.expect(client_validation_1.getValidatorImport('min')).to.eq('numeric/compare');
        chai_1.expect(client_validation_1.getValidatorImport('negative')).to.eq('numeric/negative');
        chai_1.expect(client_validation_1.getValidatorImport('negativeOrZero')).to.eq('numeric/negativeOrZero');
        chai_1.expect(client_validation_1.getValidatorImport('notBlank')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('notEmpty')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('notNull')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('null')).to.eq('null');
        chai_1.expect(client_validation_1.getValidatorImport('pattern')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('positive')).to.eq('numeric/positive');
        chai_1.expect(client_validation_1.getValidatorImport('positiveOrZero')).to.eq('numeric/positiveOrZero');
        chai_1.expect(client_validation_1.getValidatorImport('size')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('length')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('url')).to.eq(undefined);
        chai_1.expect(client_validation_1.getValidatorImport('futureOrPresent', 'date')).to.eq('temporal/futureOrPresentDate');
        chai_1.expect(client_validation_1.getValidatorImport('future', 'instant')).to.eq('temporal/futureInstant');
        chai_1.expect(client_validation_1.getValidatorImport('pastOrPresent', 'instant')).to.eq('temporal/pastOrPresentInstant');
        chai_1.expect(client_validation_1.getValidatorImport('past', 'instant')).to.eq('temporal/pastInstant');
    });
});
const MUTATION = `mutation Update_Client_Validation_Test_Entity($input: ClientValidationTestEntityInput!) {
    updateClientValidationTestEntity(input: $input) {
      id
    }
  }`;
const OWNER_MUTATION = 'mutation Update_Owner($input: OwnerInputDTO) {\n    updateOwner(input: $input) {\n      id\n    }\n  }';
const EXPECTED_RULES = {
    businessEmail: [{ rule: 'email', regexp: '.*@haulmont.com$' }],
    email: [{ rule: 'email', regexp: '.*' }],
    eulaAccepted: [
        { rule: 'assertTrue' },
        { rule: 'notNull' }
    ],
    future: [{ rule: 'future', type: 'instant' }],
    futureOrPresent: [{ rule: 'futureOrPresent', type: 'date' }],
    length: [{ rule: 'length', min: 3, max: 5 }],
    negative: [
        { rule: 'negative' },
        { rule: 'notNull' }
    ],
    negativeOrZero: [
        { rule: 'negativeOrZero' },
        { rule: 'notNull' }
    ],
    notBlank: [
        { rule: 'notBlank' },
        { rule: 'notNull' }
    ],
    notEmpty: [
        { rule: 'notEmpty' },
        { rule: 'notNull' }
    ],
    nullField: [{ rule: 'null' }],
    past: [{ rule: 'past', type: 'instant' }],
    pastOrPresent: [{ rule: 'pastOrPresent', type: 'instant' }],
    pattern: [{ rule: 'pattern', regexp: '^A.*' }],
    positive: [{ rule: 'positive' }],
    positiveOrZero: [{ rule: 'positiveOrZero' }],
    price: [
        { rule: 'decimalMax', value: '199.99', inclusive: true },
        { rule: 'decimalMin', value: '150', inclusive: false },
        { rule: 'digits', integer: 3, fraction: 2 }
    ],
    quantity: [
        { rule: 'max', value: 10000 },
        { rule: 'min', value: 500 }
    ],
    requiredWithoutDirective: [{ rule: 'notNull' }],
    size: [{ rule: 'size', min: 3, max: 5, isList: false }],
    trialExpired: [{ rule: 'assertFalse' }],
    urlString: [{ rule: 'url' }],
    urlWithoutDirective: [{ rule: 'url' }]
};
const EXPECTED_IMPORTS = [
    'assertFalse',
    'assertTrue',
    'temporal/futureInstant',
    'temporal/futureOrPresentDate',
    'temporal/pastInstant',
    'temporal/pastOrPresentInstant',
    'numeric/negative',
    'numeric/negativeOrZero',
    'numeric/positive',
    'numeric/positiveOrZero',
    'numeric/compare',
    'numeric/digits',
    'null',
];
//# sourceMappingURL=client-validation.test.js.map